@@ -11,11 +11,7 @@
 
 require 'gettext'
 
-<<<<<<< container.rb
-module GetText
-=======
 module GetText 
->>>>>>> 1.6
   # Deprecated. You don't need this. Use GetText instead.
   module Container
     include GetText
