@@ -2,11 +2,11 @@
 =begin
   rgettext.rb - Generate a .pot file.
 
-  Copyright (C) 2003-2005  Masao Mutoh
+  Copyright (C) 2003-2006  Masao Mutoh
   Copyright (C) 2001,2002  Yasushi Shoji, Masao Mutoh
  
-      Yasushi Shoji   <yashi@yashi.com>
-      Masao Mutoh     <mutoh@highway.ne.jp>
+      Yasushi Shoji   <yashi at atmark-techno.com>
+      Masao Mutoh     <mutoh at highway.ne.jp>
  
   You may redistribute it and/or modify it under the same
   license terms as Ruby.
@@ -25,19 +25,19 @@ module GetText
 
     # constant values
     VERSION = GetText::VERSION
-    DATE = %w($Date: 2006/06/11 15:36:20 $)[1]
+    DATE = %w($Date: 2008/01/27 10:58:10 $)[1]
     MAX_LINE_LEN = 70
 
     @ex_parsers = []
     [
       ["glade.rb", "GladeParser"],
       ["erb.rb", "ErbParser"],
-      ["activerecord.rb", "ActiveRecordParser"],
+      ["active_record.rb", "ActiveRecordParser"],
 #      ["ripper.rb", "RipperParser"],
       ["ruby.rb", "RubyParser"] # Default parser.
     ].each do |f, klass|
       begin
-	require File.join("gettext/parser/#{f}")
+	require "gettext/parser/#{f}"
 	@ex_parsers << GetText.const_get(klass)
       rescue
 	$stderr.puts _("'%{klass}' is ignored.") % {:klass => klass}
@@ -47,6 +47,26 @@ module GetText
 
     module_function
 
+    # Add an option parser
+    # the option parser module requires to have target?(file) and parser(file, ary) method.
+    # 
+    #  require 'gettext/rgettext'
+    #  module FooParser
+    #    module_function
+    #    def target?(file)
+    #      File.extname(file) == '.foo'  # *.foo file only.
+    #    end
+    #    def parse(file, ary)
+    #      :
+    #      return ary # [["msgid1", "foo.rb:200"], ["msgid2", "bar.rb:300", "baz.rb:400"], ...]
+    #    end
+    #  end
+    #  
+    #  GetText::RGetText.add_parser(FooParser)
+    def add_parser(klass)
+      @ex_parsers.insert(0, klass)
+    end
+
     def generate_pot_header # :nodoc:
       time = Time.now.strftime("%Y-%m-%d %H:%M")
       off = Time.now.utc_offset
@@ -103,6 +123,41 @@ msgstr ""
       str
     end
 
+    def normalize(ary)  # :nodoc:
+      used_plural_msgs = []
+      ary.select{|item| item[0].include? "\000"}.each do |plural_msg|
+        next if used_plural_msgs.include?(plural_msg[0])
+
+        key = plural_msg[0].split("\000")[0]
+
+        ary.dup.each do |single_msg|
+          if single_msg[0] == key or
+              (single_msg[0] =~ /^#{Regexp.quote(key)}\000/ and 
+               single_msg[0] != plural_msg[0])
+            if single_msg[0] != key 
+              warn %Q[Warning: n_("#{plural_msg[0].gsub(/\000/, '", "')}") and n_("#{single_msg[0].gsub(/\000/, '", "')}") are duplicated. First msgid was used.] 
+                used_plural_msgs << single_msg[0]
+            end
+
+            single_msg[1..-1].each do |line_info|
+              plural_msg << line_info
+              fname = line_info.split(/:/)[0]
+              sorted = plural_msg[1..-1].select{|l| l.split(/:/)[0].include?(fname)}.collect{|l| 
+                aline = l.split(/:/)
+                [aline[0], aline[1] =~ /[0-9]+/ ? aline[1].to_i : aline[1]]
+              }.sort.collect{|l| "#{l[0]}:#{l[1]}"}
+              plural_msg.delete_if{|i| sorted.include?(i)}
+              sorted.each {|i|
+                plural_msg << i
+              }
+            end
+            ary.delete(single_msg)
+          end
+        end
+      end
+      ary.collect{|i| i.uniq}
+    end
+
     def parse(files) # :nodoc:
       ary = []
       files.each do |file|
@@ -114,19 +169,18 @@ msgstr ""
 	    end
 	  end
 	rescue
-	  puts $!
-	  exit 1
+	  puts "Error occurs in " + file
+	  raise
 	end
       end
-
-      ary
+      normalize(ary)
     end
 
     def check_options # :nodoc:
       output = STDOUT
 
       opts = OptionParser.new
-      opts.banner = _("Usage: %s input.rb [-o output.pot]") % $0
+      opts.banner = _("Usage: %s input.rb [-r parser.rb] [-o output.pot]") % $0
       opts.separator("")
       opts.separator(_("Extract translatable strings from given input files."))
       opts.separator("")
@@ -136,11 +190,19 @@ msgstr ""
 	unless FileTest.exist? out
 	  output = File.new(File.expand_path(out), "w+")
 	else
-	  $stderr.puts(_("File '%s' has already existed.") % out)
+	  $stderr.puts(_("File '%s' already exists.") % out)
 	  exit 1
 	end
       end
 
+      opts.on("-r", "--require=library", _("require the library before executing rgettext")) do |out|
+	require out
+      end
+
+      opts.on("-d", "--debug", _("run in debugging mode")) do
+	$DEBUG = true
+      end
+
       opts.on_tail("--version", _("display version information and exit")) do
 	puts "#{$0} #{VERSION} (#{DATE})"
 	puts "#{File.join(Config::CONFIG["bindir"], Config::CONFIG["RUBY_INSTALL_NAME"])} #{RUBY_VERSION} (#{RUBY_RELEASE_DATE}) [#{RUBY_PLATFORM}]"
