@@ -394,7 +394,7 @@ module GetText
     
     # constant values
     VERSION = GetText::VERSION
-    DATE = %w($Date: 2006/06/11 15:36:20 $)[1]
+    DATE = %w($Date: 2007/07/21 15:03:05 $)[1]
     
     module_function
 
@@ -459,7 +459,7 @@ module GetText
       
       m = Merger.new
       result = m.merge(defpo, refpot)      
-      pp result if $DEBUG
+      p result if $DEBUG
       print result.generate_po if $DEBUG
       
       begin
