@@ -7,16 +7,26 @@
   license terms as Ruby.
 =end
 
+require 'rbconfig'
+if /mingw|mswin|mswin32/ =~ RUBY_PLATFORM
+  ENV['PATH'] = %w(bin lib).collect{|dir|
+    "#{Config::CONFIG["prefix"]}\\lib\\GTK\\#{dir};"
+  }.join('') + ENV['PATH']
+end
+
 require 'gettext/rgettext'
 require 'gettext/rmsgfmt'
 require 'fileutils'
 
 module GetText
+  bindtextdomain "rgettext"
+
   module_function
 
+
   # Merges two Uniforum style .po files together. 
   #
-  # *Note* "msgmerge" tool is included in GNU GetText. So you need to install GNU GetText. 
+  # *Note* This function requires "msgmerge" tool included in GNU GetText. So you need to install GNU GetText. 
   #
   # The def.po file is an existing PO file with translations which will be taken 
   # over to the newly created file as long as they still match; comments will be preserved,
@@ -40,6 +50,10 @@ module GetText
 
     cont = ""
     if FileTest.exist? defpo
+      `#{cmd} --version`
+      unless $?.success?
+        raise _("`#{cmd}' may not be found. \nInstall GNU Gettext then set PATH or MSGMERGE_PATH correctly.")
+      end
       cont = `#{cmd} #{defpo} #{refpo}`
     else
       File.open(refpo) do |io| 
@@ -49,8 +63,9 @@ module GetText
     if cont.empty?
       failed_filename = refpo + "~"
       FileUtils.cp(refpo, failed_filename)
-      $stderr.puts _("Failed to merge with %{defpo} - skipping!") % {:defpo => defpo}
-      $stderr.puts _("Please check new .pot in %{failed_filename}") %{:failed_filename => failed_filename}
+      $stderr.puts _("Failed to merge with %{defpo}") % {:defpo => defpo}
+      $stderr.puts _("New .pot was copied to %{failed_filename}") %{:failed_filename => failed_filename}
+      raise _("Check these po/pot-files. It may have syntax errors or something wrong.")
     else
       cont.sub!(/(Project-Id-Version\:).*$/, "\\1 #{app_version}\\n\"")
       File.open(defpo, "w") do |out|
