@@ -8,7 +8,7 @@ require 'libglade2'
 
 class HelloLibglade2
   def initialize(path, appname)
-    @glade = GladeXML.new(path, nil, appname) {|handler| method(handler)}
+    @glade = GladeXML.new(path, nil, appname, "locale") {|handler| method(handler)}
   end
   def on_quit
     puts "Hello world"
