@@ -1,3 +1,3 @@
-require 'gettext/rmsgfmt'
+require 'gettext/utils'
 
 GetText.create_mofiles(true, "po", "locale")
